/*
avr-isp.h
	
avr-isp: Programmer software for Atmel AVR family.
Copyright (C) 2001 Peter van den Bosch
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author:
	Peter van den Bosch
	pvdb@dse.nl
	www.dse.nl/~pvdb
*/

#include <sys/times.h>

/* Version information */
#define		VERSION_MAJOR		0
#define		VERSION_MINOR		3
#define		VERSION_RELEASE	"-test1"

/* AVR support */
struct ID_CODES {
	unsigned int	ID;
	char		device[ 12];
	int		FLASHsize;              /* FLASH size in BYTES */
	int		EEPROMsize;
	unsigned char	P1, P2, P3;
   int      pageprog;               /* Page size in WORDS */
   char     (*fusename_l)[ 9], (*fusename_h)[ 9];
};

extern struct ID_CODES id_codes[];
extern struct ID_CODES *this_device;

extern int FLAG_ENDIAN;
extern int FLAG_HELP;
extern int FLAG_LIST;
extern int FLAG_VERSION;
extern int FLAG_NOVERIFY;
extern int FLAG_READAVR;
extern int FLAG_WRITEAVR;
extern int FLAG_WRITEFUSE;
extern int FLAG_READFUSE;
extern int FLAG_WRITELOCK;

extern char STRFUSE_L1[ 8][ 9];
extern char STRFUSE_H1[ 8][ 9];
extern char STRFUSE_H2[ 8][ 9];
extern char STRFUSE_H3[ 8][ 9];

/* Definition to discern between FLASH and EEPROM programming */
#define FLASH	0
#define EEPROM	1


/* ===================
   Hardware definition 
   =================== */
/* Base IO address for parallel port (LPT1 normally 0x378) */
#define		LPT_BASE		0x378

/* Pin definitions for the various functions. 
	SCK	Clock signal
	MOSI	Input for writing to device
	MISO	Output for reading from device
	RESET	Keep device in reset while writing */
#define DONTRONICS
#ifdef DONTRONICS
#define		RESET	1		/* at LPT_BASE */
#define		SCK	2		/* at LPT_BASE */
#define		MOSI	3		/* at LPT_BASE */
#define		MISO	6		/* at LPT_BASE+1 */
#undef      REVMISO
#else
#define		RESET	2		/* at LPT_BASE */
#define		SCK	0		/* at LPT_BASE */
#define		MOSI	1		/* at LPT_BASE */
#define		MISO	5		/* at LPT_BASE+1 */
#undef      REVMISO
#endif

#define TCK_CLK   100

/* Function definitions, bodies are in avr-isp.cc */

#ifdef __cplusplus
extern "C" {
#endif
int avr_openport( void);
unsigned char sendByte( unsigned char);
int avr_init( void);
int avr_readID( struct ID_CODES **);
int avr_run( void);
int avr_erase( void);
int avr_readfuses( unsigned char *);
int avr_writefuses( unsigned char *);
int avr_program (FILE *, FILE *, void *(*)( int, double));
int avr_read( char *, unsigned int, unsigned int, int);
int download( int, unsigned char *, int, unsigned int);
int convert( int, char *, unsigned char *, int *);
void terminate( int, char*);
void waitms( clock_t);
#ifdef __cplusplus
}
#endif

#define	not_OK( str)	{ fprintf( stderr, str); return -1; }
#define hex2dec( a, b)	( ((a<='9')?a-'0':a-'A'+10)*16+((b<='9')?b-'0':b-'A'+10) )

#define		HELPMSG		"Usage: avr-isp [options] [--flash filename][--eeprom filename] \n\
\navr-isp downloads the program from an Intel HEX formatted file \n\
into the flash memory of an Atmel AVR uC.\n\
\nOptions:\n\
--flash f       Programs Intel Hex formatted file in flash program memory\n\
--eeprom f      Programs Intel Hex formatted file in EEPROM memory\n\n\
--help          Displays this message.\n\
--endian        Write high bytes to low address and vice-versa.\n\
--noverify      Disable verification after programming (faster).\n\n\
--list          List of supported u-controllers.\n\
--version	Display version information. \n\n\
--read          This flag will READ flash and eeprom from the AVR and write to standard output.\n\
--readfuse      Read the fuse bytes.\n\
--writefuse l h Write the low and high fuse bytes.\n\
--writelock l   Write the lock byte.\n\
\n\n\nNotes:\n\
If you used Atmels AVR Studio and assembler is is very likely you need \n\
program with the --endian option. The AVA assembler (v0.3b) works fine \n\
without that flag (depending on your settings of course).\n\n"

#define  ABOUTMSG    "gAVR 0.1 \n\n"\
"Released under GPL license, blablabla...\n\n"\
"Peter van den Bosch\npvdb@dse.nl\npvdb.dse.nl\n\n"
