/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <unistd.h>

#include "interface.h"
#include "support.h"

#include "../include/avr-isp.h"
#include "../include/gui.h"

GtkWidget   *mainwindow;

int
gui_init (void)
{
   GtkWidget *tmp;
   int i;

   gtk_set_locale ();
   gtk_init (NULL, NULL);

   add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");

   mainwindow = create_mainwindow ();
   gtk_widget_show (mainwindow);
   
   /* Make endianess consistent */
   tmp = lookup_widget (GTK_WIDGET (mainwindow), "togglebuttonEndian");
   if (FLAG_ENDIAN)
   {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), TRUE);
   }
   else
   {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), FALSE);
   }

   /* Fill the drop-box with supported AVR-types */
   tmp = lookup_widget (GTK_WIDGET (mainwindow), "comboboxDevice");
   for (i = 0; id_codes[i].ID; i++)
   {
      gtk_combo_box_append_text( GTK_COMBO_BOX( tmp), id_codes[ i].device);
   }
   gtk_combo_box_set_active( GTK_COMBO_BOX( tmp), 0);

   /* Try to initialize the parallel port, if it doesn't work
    * then show an error and exit */
   if (-1 == avr_openport ())
   {
#ifndef FAKE_PORT
      GtkWidget *msg;

      msg = gtk_message_dialog_new (GTK_WINDOW (mainwindow),
                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_ERROR,
                                    GTK_BUTTONS_CLOSE,
                                    "Error opening parallel port 0x%X for usage!\n"
                                    "(you probably need root-privileges)\n",
                                    LPT_BASE);
      gtk_dialog_run (GTK_DIALOG (msg));
      gtk_widget_destroy (msg);
      return -1;
#endif
   }

      avr_run();
   gtk_main ();
   return 0;
}

void
show_error( char *msg)
{
      GtkWidget *msgdlg;

      msgdlg = gtk_message_dialog_new (GTK_WINDOW (mainwindow),
                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_ERROR,
                                    GTK_BUTTONS_CLOSE,
                                    msg);
      gtk_dialog_run (GTK_DIALOG (msgdlg));
      gtk_widget_destroy (msgdlg);
}

void *
update_progress ( int type, double fraction)
{
   GtkWidget *progress = lookup_widget (mainwindow, "progressbarFLASH");
   
   if( type == FLASH)
   {
      gtk_progress_bar_set_text( GTK_PROGRESS_BAR( progress), "FLASH");
   } else {
      gtk_progress_bar_set_text( GTK_PROGRESS_BAR( progress), "EEPROM");
   }
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress), fraction);
   while (gtk_events_pending ())
   {
      gtk_main_iteration ();
   }
   
   return NULL;
}
