/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include "../include/hexdata.h"

hexdata::hexdata( void)
{
   hexdata( 256*265);
}

hexdata::hexdata (int psize)
{
   data = new unsigned char[psize];
   usage = new unsigned int[psize/16+1];

   if ((data == NULL) || (usage == NULL))
   {
      fprintf (stderr, "CANNOT ALLOCATE SUFFICIENT MEMORY!\n");
   }
   else
   {
      size = psize;
      clear ();
   }
}

hexdata::~hexdata (void)
{
   if (data != NULL)
      delete data;
   if (usage != NULL)
      delete usage;
}

int
hexdata::getdata (long address)
{

   /* Returns the data on the adress. If adress is notused, 0x100 is returned.
    * 
    */

   if (isused (address))
      return data[address];
   else
      return 0x100;

}


/* Writes the data to address. If data is overwritten, 1 is returned.
 * 
 * Normally 0 is returned. Masks the address as used.
 * 
 */
int
hexdata::setdata (long address, int pdata)
{
   int bit, mask, ret;

   if( address >= size)
         return -1;
   
   data[address] = pdata;

   bit = address & 15;
   address >>= 4;
   for (mask = 1; bit > 0; bit--)
      mask <<= 1;
   if (usage[address] & mask)
   {
      ret = 1;
      printf( "Already in use: %d\n", address);
   } else {
      ret = 0;
   }
   usage[address] |= mask;

   return ret;
}


/* Checks if an addresss is used. Return 1 if used, 0 if unused.
 * 
 */
int
hexdata::isused (long address)
{
   int bit, mask;

   if( address >= size)
      return 0;
   
   bit = address & 15;
   address >>= 4;
   for (mask = 1; bit > 0; bit--)
      mask <<= 1;

   if (usage[address] & mask)
      return 1;
   else
      return 0;
}



void
hexdata::clear (void)
{
   int i;

   for (i = 0; i < size/16+1; i++)
      usage[i] = 0;
}


/* Load data from an intel-hex file.
 * 
 * 0=ERROR, >0 bytes read
 * 
 */
int
hexdata::loadihex (FILE *fp)
{
   unsigned long address;
   int cnt, indx;
   char buf[128];
   int   size = 0;

   while (fscanf (fp, "%s", buf) == 1)
   {
      if ((buf[7] == '0') && (buf[8] == '0'))
      {
         address = hex2dec (buf[3], buf[4]) * 0x100 + hex2dec (buf[5], buf[6]);
         cnt = hex2dec (buf[1], buf[2]);
         for (indx = 9; cnt > 0; cnt--)
         {
            if( -1 == setdata (address, hex2dec (buf[indx], buf[indx + 1])))
            {
               return 0;
            }
            address++;
            indx += 2;
            size++;
         }
      }
   }

   return size;
}
